package org.artifactory.request;

import org.artifactory.security.access.AccessTokenAuthentication;
import org.jfrog.access.token.JwtAccessToken;
import org.mockito.Mockito;
import org.testng.annotations.DataProvider;
import org.testng.annotations.Test;

import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;

import static org.apache.http.HttpHeaders.USER_AGENT;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;
import static org.testng.Assert.assertEquals;
import static org.testng.Assert.assertTrue;

/**
 * @author Shay Bagants
 */
@Test
public class RequestResponseHelperTest {

    @Test(dataProvider = "provideUserNames", description = "RTFACT-20177")
    public void testGetUserAgentIfXrayUser(String userName, String agent) {
        Request request = mock(Request.class);
        when(request.getHeader("User-Agent")).thenReturn(agent);
        String result = RequestResponseHelper.getUserAgentIfXrayUser(request, userName, null);
        assertEquals(result, agent);
    }

    @Test(dataProvider = "provideUserNamesForXrayToken", description = "RTFACT-20177")
    public void testGetUserAgentIfXrayUserUsingAuthenticationToken(String subject, String agent) {
        AccessTokenAuthentication accessTokenAuthentication = Mockito.mock(AccessTokenAuthentication.class);
        JwtAccessToken jwtAccessToken = Mockito.mock(JwtAccessToken.class);
        when(accessTokenAuthentication.getAccessToken()).thenReturn(jwtAccessToken);
        when(jwtAccessToken.getSubject()).thenReturn(subject);
        Request request = mock(Request.class);
        when(request.getHeader("User-Agent")).thenReturn(agent);
        String result = RequestResponseHelper.getUserAgentIfXrayUser(request, "", accessTokenAuthentication);
        assertEquals(result, agent);
    }

    @Test(dataProvider = "extractRequestClientTypeProvider")
    public void testExtractRequestClientType(String userAgent, RequestClientType expectedType) {
        // setup
        Enumeration<String> enumeration = mock(Enumeration.class);
        Iterator<String> iterator = List.of(userAgent).iterator();
        when(enumeration.asIterator()).thenReturn(iterator);

        ArtifactoryRequest request = mock(ArtifactoryRequest.class);
        when(request.getHeaderValues(USER_AGENT)).thenReturn(enumeration);

        // test
        RequestClientType requestClientType = RequestResponseHelper.extractRequestClientType(request);

        // verify
        assertEquals(requestClientType, expectedType);
    }

    @Test
    public void testExtractRequestClientTypeNoUserAgent() {
        // setup
        ArtifactoryRequest request = mock(ArtifactoryRequest.class);
        when(request.getHeaderValues(USER_AGENT)).thenReturn(null);

        // test
        RequestClientType requestClientType = RequestResponseHelper.extractRequestClientType(request);

        // verify
        assertEquals(requestClientType, RequestClientType.UNKNOWN);
    }

    @Test(dataProvider = "clientSupportsM3SnapshotVersionsProvider")
    public void testClientSupportsM3SnapshotVersions(String userAgent, boolean expectedSupport) {
        // setup
        Enumeration<String> enumeration = mock(Enumeration.class);
        Iterator<String> iterator = List.of(userAgent).iterator();
        when(enumeration.asIterator()).thenReturn(iterator);

        ArtifactoryRequest request = mock(ArtifactoryRequest.class);
        when(request.getHeaderValues(USER_AGENT)).thenReturn(enumeration);
        when(request.getHeader(USER_AGENT)).thenReturn(userAgent);

        // test
        boolean supports = RequestResponseHelper.clientSupportsM3SnapshotVersions(request);

        // verify
        assertEquals(supports, expectedSupport);
    }

    @Test
    public void testClientSupportsM3SnapshotVersionsNoUserAgent() {
        // setup
        ArtifactoryRequest request = mock(ArtifactoryRequest.class);
        when(request.getHeaderValues(USER_AGENT)).thenReturn(null);
        when(request.getHeader(USER_AGENT)).thenReturn(null);

        // test
        boolean supports = RequestResponseHelper.clientSupportsM3SnapshotVersions(request);

        // verify
        assertTrue(supports);
    }

    @DataProvider
    public static Object[][] clientSupportsM3SnapshotVersionsProvider() {
        return new Object[][] {
                //header,  type
                {"Debian APT-HTTP/1.3 (2.0.2ubuntu0.1)", true},
                {"Apache Ivy/", false},
                {"Wharf Ivy/", false},
                {"anything", true},
                {"", true},
        };
    }

    @DataProvider
    public static Object[][] extractRequestClientTypeProvider() {
        return new Object[][] {
                //header,  type
                {"Debian APT-HTTP/1.3 (2.0.2ubuntu0.1)", RequestClientType.DEBIAN},
                {"Apache Ivy/", RequestClientType.APACHE_IVY},
                {"Wharf Ivy/", RequestClientType.WHARF_IVY},
                {"anything", RequestClientType.UNKNOWN},
                {"", RequestClientType.UNKNOWN},
        };
    }

    @DataProvider
    public static Object[][] provideUserNames() {
        return new Object[][] {
                //user,  agent
                {"xray", "xray-agent"},
                {"token:jfrt@123", ""},
                {"token:xyz@123", ""},
                {"token:abc", ""},
                {"xday", ""},
                {"admin", ""},
                {"system", ""},
                {"me", ""}
        };
    }

    @DataProvider
    public static Object[][] provideUserNamesForXrayToken() {
        return new Object[][] {
                //user,  agent
                {"jfxr@123", "xray-token"},
                {"xray", ""},
                {"jfrt@123", ""},
                {"token:xyz@123", ""},
                {"abc", ""},
                {"xday", ""},
                {"admin", ""},
                {"system", ""},
                {"me", ""},
                {"jfxana@123", "xray-token"},
                {"jfxpst@123", "xray-token"},
                {"jfxidx@123", "xray-token"},
        };
    }
}