#!/bin/bash
#
# Startup script for Artifactory Metadata
#

getPid() {
    # Identify process which is running metadata binary
    export myPID=$(ps -ef | grep -i "${METADATA_BIN}" | grep -v grep | awk -F' ' '{print $2}' 2>/dev/null)
}

## configure on which port artifactory (access) is running to connect to..
setUpConnection(){
    ## Only set this var if not already set
    getSystemValue "artifactory.port" "8081"
    export ARTIFACTORY_PORT=${YAML_VALUE}

    ACCESS_XML=$JF_PRODUCT_HOME/app/artifactory/tomcat/conf/Catalina/localhost/access.xml
    if [[ -f "$ACCESS_XML" ]];then
        export ACCESS_PATH=$(grep path $ACCESS_XML | awk '{ print $2}'  | cut -d'=' -f2 | sed -e 's|/||' -e 's|"||g')
        if [ -z $ACCESS_PATH ]; then
            export ACCESS_PATH="access"
        fi
    else
        export ACCESS_PATH="access"
    fi
    export JF_METADATA_ACCESSCLIENT_URL=${JF_METADATA_ACCESSCLIENT_URL:-"http://localhost:$ARTIFACTORY_PORT/$ACCESS_PATH"}
}

exitError() {
    echo -e "\n\033[31mERROR: $1\033[0m\n"

    [ -z "${2}" ] || usage
    exit 1
}

ARTIFACTORY_BIN_FOLDER="$(cd $(dirname "${BASH_SOURCE[0]}")/../../bin && pwd)"
artDefaultFile="${ARTIFACTORY_BIN_FOLDER}/artifactory.default"

# used in new structure
METADATA_NAME=metadata
NAME=Metadata

SCRIPT_DIR="$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)"
SCRIPT_NAME=${METADATA_NAME}.sh

. ${artDefaultFile} || exitError "ERROR: $artDefaultFile does not exist or not executable"

: ${JF_PRODUCT_HOME:="$(cd ${ARTIFACTORY_BIN_FOLDER}/../.. && pwd)"}

WAIT_FOR_ARTIFACTORY_LIMIT=180

METADATA_PID=${JF_PRODUCT_HOME}/app/run/${METADATA_NAME}.pid
METADATA_BIN=

usage() {
    cat << END_USAGE

${SCRIPT_NAME} - script for controlling ${METADATA_NAME}

Usage: ./${SCRIPT_NAME} <action>

action:    help|start|stop|restart|status


END_USAGE

    exit 1
}

# Process command line options. See usage above for supported options
processOptions () {
    # Get action - help|start|stop|restart|status
    ACTION=${1}
    shift

    if [[ ! "${ACTION}" =~ ^(help|start|stop|restart|status)$ ]]; then
        usage
    fi

    if [[ ! "${ACTION}" =~ ^(help|start|stop|restart|status)$ ]]; then
        usage
    fi

    if [[ "${ACTION}" =~ ^help$ ]]; then
        usage
    fi

    # Process the options
    while [[ $# > 0 ]]; do
        case "$1" in
            *)
                usage
            ;;
        esac
    done

}

sourceScript(){
    local fileName=$1

    [ ! -z "${fileName}" ] || errorExit "Target file is not set"
    [   -f "${fileName}" ] || errorExit "${fileName} file is not found"
    source "${fileName}"   || errorExit "Unable to source ${fileName}, please check if the $USER user has permissions to perform this action"
}

initHelpers(){
    local systemYamlHelper="${ARTIFACTORY_BIN_FOLDER}"/systemYamlHelper.sh
    local installerCommon="${ARTIFACTORY_BIN_FOLDER}"/installerCommon.sh

    export YQ_PATH="${ARTIFACTORY_BIN_FOLDER}/../third-party/yq"
    sourceScript "${systemYamlHelper}"

    sourceScript "${installerCommon}"

    # init at each service startup META
    export JF_SYSTEM_YAML="${JF_PRODUCT_HOME}/var/etc/system.yaml"
}

setExecutable() {
    local os=$(uname -s)
    local arch=$(uname -m)

    # Resolve OS
    case ${os} in
        Linux)
            os=linux
        ;;
        Darwin)
            os=darwin
        ;;
        *)
            exitError "OS type ${os} is not supported"
        ;;
    esac

    # Resolve cpu Architecture
    case ${arch} in
        x86_64)
            arch=amd64
        ;;
        *)
            exitError "CPU architecture ${arch} is not supported"
        ;;
    esac

    METADATA_BIN="${SCRIPT_DIR}/jf-metadata"
}

isRunning() {
    local ps=
    # Set binary to run
    setExecutable
    local bin=${METADATA_BIN}
    # Check running state by PID
    if [ -e "${METADATA_PID}" ]; then
        PID_VALUE=$(cat ${METADATA_PID})
        if [ -n "${PID_VALUE}" ]; then
            unset ps
            getPid
            if [[ ! -z "${myPID}" ]];then
                export ps="${myPID}"
            fi
        fi
    else
        # Try and find by process
        ps=$(ps -ef | grep -i ${bin} | grep -v grep 2> /dev/null)
        PID_VALUE=$(echo -n ${ps} | awk '{print $2}')
    fi
    if [ -z "${ps}" ]; then
        if [ -z "${PID_VALUE}" ]; then
            return 1 # Not running
        else
            return 2 # PID exists but process dead
        fi
    else
        return 0 # Is running
    fi
}

start() {
    echo "Starting ${METADATA_NAME}..."

    # Check if running
    isRunning
    case $? in
        0)
            echo "${METADATA_NAME} is already running (PID: ${PID_VALUE})"
            restart
            return 0
        ;;
        1)
#            echo "${METADATA_NAME} not running. Proceed to start it up."
        ;;
        2)
            echo "PID exists (${PID_VALUE}), but ${METADATA_NAME} not running. Proceed to start it up."
        ;;
        *)
            exitError "Function isRunning() returned an unknown value ($?)"
        ;;
    esac

    # Start metadata
    if [ -n "$METADATA_PID" ];
    then
        mkdir -p $(dirname "$METADATA_PID") || \
        errorExit "Could not create dir for $METADATA_PID";
    fi

    # Run from var folder to avoid nohup errors
    cd ${JF_PRODUCT_HOME}/var

    setUpConnection
    exportEnv "shared"
    exportEnv "${METADATA_NAME}"

    echo "JF_METADATA_ACCESSCLIENT_URL: ${JF_METADATA_ACCESSCLIENT_URL}"
    
    if [ $(isRunningInsideAContainer)  == "$FLAG_Y" ]; then
        if $(isConsoleLogDisabled >/dev/null 2>&1); then
            ${METADATA_BIN} &
        else
            ${METADATA_BIN} > >(tee >(redirectServiceLogsToFile)) 2>&1 &
        fi
    else
        if $(isConsoleLogDisabled >/dev/null 2>&1); then
            ${METADATA_BIN} >/dev/null 2>&1 &
        else
            ${METADATA_BIN} > >(redirectServiceLogsToFile ) 2>&1 &
        fi
    fi
    local pid=$!
    echo -n ${pid} > ${METADATA_PID}

    echo "${METADATA_NAME} started. PID: ${pid}"
}


stop() {
    echo "Stopping ${METADATA_NAME}..."

    # Check if running
    isRunning
    case $? in
        0)
            echo "${METADATA_NAME} is running (PID: ${PID_VALUE}). Stopping it..."
        ;;
        1)
            echo "${METADATA_NAME} not running."
            return 0
        ;;
        2)
            echo "PID exists (${PID_VALUE}), but ${METADATA_NAME} not running. Removing PID file..."
            rm -f ${METADATA_PID}
            return 0
        ;;
        *)
            exitError "Function isRunning() returned an unknown value ($?)"
        ;;
    esac

    # Kill using PID
    kill ${PID_VALUE} || exitError "Stopping ${METADATA_NAME} failed"
    rm -f ${METADATA_PID}
    echo "${METADATA_NAME} stopped"
}

restart() {
    echo "Restarting ${METADATA_NAME}..."
    ${SCRIPT_DIR}/${SCRIPT_NAME} stop
    # Breathing space after stop - its too fast
    sleep 1
    ${SCRIPT_DIR}/${SCRIPT_NAME} start $*
}

status() {
    echo -n "${METADATA_NAME} is "

    # Check if running
    isRunning
    case $? in
        0)
            echo "running (PID: ${PID_VALUE})"
        ;;
        1)
            echo "not running"
        ;;
        2)
            echo "PID exists (${PID_VALUE}), but ${METADATA_NAME} not running."
        ;;
        *)
            exitError "Function isRunning() returned an unknown value ($?)"
        ;;
    esac
}

initHelpers
processOptions $*
eval ${ACTION}
