package org.artifactory.storage.db.event.error.itest.dao;

import com.google.common.collect.Lists;
import org.artifactory.storage.db.event.error.dao.NodeEventsErrorsDAO;
import org.artifactory.storage.db.event.error.model.NodeEventsErrorsDBO;
import org.artifactory.storage.db.itest.DbBaseTest;
import org.artifactory.storage.event.EventType;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.jdbc.core.namedparam.SqlParameterSource;
import org.springframework.jdbc.core.namedparam.SqlParameterSourceUtils;
import org.springframework.jdbc.core.simple.SimpleJdbcInsert;
import org.testng.Assert;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import javax.sql.DataSource;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Comparator;
import java.util.List;

import static org.artifactory.storage.db.itest.DbTestUtils.createSimpleJdbcInsert;

/**
 * @author Mor Merhav
 */
@Test
public class NodeEventsErrorsDAOTest extends DbBaseTest {
    private static final String TABLE_NODE_EVENTS_ERRORS = "node_events_errors";
    @Autowired
    private NodeEventsErrorsDAO dao;
    @Autowired
    private DataSource dataSource;

    private JdbcTemplate jdbcTemplate;
    private SimpleJdbcInsert simpleJdbcInsert;

    @BeforeClass
    public void setup() {
        jdbcTemplate = new JdbcTemplate(dataSource);
        simpleJdbcInsert = createSimpleJdbcInsert(jdbcTemplate.getDataSource())
                .withTableName(TABLE_NODE_EVENTS_ERRORS);
    }

    @AfterMethod
    public void cleanUp() {
        jdbcTemplate.execute("TRUNCATE TABLE " + TABLE_NODE_EVENTS_ERRORS);
    }

    @Test
    public void testSelectAll() throws SQLException {
        NodeEventsErrorsDBO dbo1 = create(1, null, "EventCategory");
        NodeEventsErrorsDBO dbo2 = create(2, "opKey", "EventCategory");

        SqlParameterSource[] batch = SqlParameterSourceUtils.createBatch(dbo1, dbo2);
        simpleJdbcInsert.executeBatch(batch);

        List<NodeEventsErrorsDBO> expectedNodeEventErrorsDBOs = Lists.newArrayList(dbo2, dbo1);
        expectedNodeEventErrorsDBOs.sort(Comparator.comparing(NodeEventsErrorsDBO::getEventTime)
                .thenComparing(NodeEventsErrorsDBO::getErrorId));

        List<NodeEventsErrorsDBO> nodeEventsErrorsDBOS = dao.selectAll("EventCategory");

        Assert.assertEquals(expectedNodeEventErrorsDBOs, nodeEventsErrorsDBOS);
    }

    @Test
    public void testSelect() throws SQLException {
        NodeEventsErrorsDBO dbo1 = create(1, null, EventType.create, "EventCategory1", "Path1");
        NodeEventsErrorsDBO dbo2 = create(2, "opKey", EventType.create, "EventCategory2", "Path3");
        NodeEventsErrorsDBO dbo3 = create(3, "opKey", EventType.delete, "EventCategory2", "Path3");
        NodeEventsErrorsDBO dbo4 = create(4, "opKey2", EventType.props, "EventCategory3", "Path4");
        List<NodeEventsErrorsDBO> dbos = Lists.newArrayList(dbo1, dbo2, dbo3, dbo4);

        SqlParameterSource[] batch = SqlParameterSourceUtils.createBatch(dbos);
        simpleJdbcInsert.executeBatch(batch);

        NodeEventsErrorsDBO returnedDbo1 = dao.select("EventCategory1", EventType.create, "Path1");
        NodeEventsErrorsDBO returnedDbo3 = dao.select("EventCategory2", EventType.delete, "Path3", "opKey");

        Assert.assertNotNull(returnedDbo1);
        Assert.assertEquals(dbo1, returnedDbo1);

        Assert.assertNotNull(returnedDbo3);
        Assert.assertEquals(dbo3, returnedDbo3);
    }

    @Test
    public void testUpdate() throws SQLException {
        NodeEventsErrorsDBO dbo1 = create(1, null,
                EventType.create, "EventCategory1", "Path1", 0, 2, "defaultErrMsg");
        NodeEventsErrorsDBO dbo2 = create(2, "opKey",
                EventType.create, "EventCategory2", "Path3", 0, 2, "defaultErrMsg");
        NodeEventsErrorsDBO dbo3 = create(3, "opKey",
                EventType.delete, "EventCategory2", "Path3", 0, 2, "defaultErrMsg");
        NodeEventsErrorsDBO dbo4 = create(4, "opKey2",
                EventType.props, "EventCategory3", "Path4", 0, 2, "defaultErrMsg");
        List<NodeEventsErrorsDBO> dbos = Lists.newArrayList(dbo1, dbo2, dbo3, dbo4);

        SqlParameterSource[] batch = SqlParameterSourceUtils.createBatch(dbos);
        simpleJdbcInsert.executeBatch(batch);

        NodeEventsErrorsDBO updatedDbo1 = createUpdated(dbo1, 1, 3, "newErrMsg");
        NodeEventsErrorsDBO updatedDbo2 = createUpdated(dbo2, 1, 3, "newErrMsg");

        int numRowsAffected1 = dao.update(updatedDbo1);
        Assert.assertEquals(1, numRowsAffected1);
        int numRowsAffected2 = dao.update(updatedDbo2);
        Assert.assertEquals(1, numRowsAffected2);

        NodeEventsErrorsDBO returnedDbo1 = jdbcTemplate
                .queryForObject("SELECT * FROM " + TABLE_NODE_EVENTS_ERRORS + " WHERE error_id = " + dbo1.getErrorId(),
                        getRowMapper());
        NodeEventsErrorsDBO returnedDbo2 = jdbcTemplate
                .queryForObject("SELECT * FROM " + TABLE_NODE_EVENTS_ERRORS + " WHERE error_id = " + dbo2.getErrorId(),
                        getRowMapper());

        Assert.assertEquals(updatedDbo1, returnedDbo1);
        Assert.assertEquals(updatedDbo2, returnedDbo2);
    }

    @Test
    public void testInsert() throws SQLException {
        NodeEventsErrorsDBO dbo1 = create(1, null, "EventCategory");
        NodeEventsErrorsDBO dbo2 = create(2, "opKey", "EventCategory");

        List<NodeEventsErrorsDBO> dbos = Lists.newArrayList(dbo1, dbo2);
        dao.insert(dbos);

        List<NodeEventsErrorsDBO> returnedDBOs =
                jdbcTemplate.query("SELECT * FROM " + TABLE_NODE_EVENTS_ERRORS, getRowMapper());

        dbos.sort(Comparator.comparing(NodeEventsErrorsDBO::getErrorId));
        returnedDBOs.sort(Comparator.comparing(NodeEventsErrorsDBO::getErrorId));

        Assert.assertEquals(dbos, returnedDBOs);
    }

    @Test
    public void testSelectErrorsByOperationKey() throws SQLException {
        NodeEventsErrorsDBO dbo1 = create(1, null, "EventCategory");
        NodeEventsErrorsDBO dbo2 = create(2, "opKey", "EventCategory");
        NodeEventsErrorsDBO dbo3 = create(3, "opKey", "EventCategory");
        NodeEventsErrorsDBO dbo4 = create(4, "opKey2", "EventCategory");
        List<NodeEventsErrorsDBO> dbos = Lists.newArrayList(dbo1, dbo2, dbo3, dbo4);

        SqlParameterSource[] batch = SqlParameterSourceUtils.createBatch(dbos);
        simpleJdbcInsert.executeBatch(batch);

        List<NodeEventsErrorsDBO> returnedDbos = Lists.newArrayList(dao.select("EventCategory", "opKey"));
        Assert.assertNotNull(returnedDbos);
        Assert.assertEquals(2, returnedDbos.size());

        returnedDbos.sort(Comparator.comparing(NodeEventsErrorsDBO::getErrorId));

        Assert.assertEquals(dbo2, returnedDbos.get(0));
        Assert.assertEquals(dbo3, returnedDbos.get(1));
    }

    @Test
    public void testDelete() throws SQLException {
        NodeEventsErrorsDBO dbo1 = create(1, null,
                EventType.create, "EventCategory1", "Path1", 0, 2, "defaultErrMsg");
        NodeEventsErrorsDBO dbo2 = create(2, "opKey",
                EventType.create, "EventCategory2", "Path2", 0, 2, "defaultErrMsg");
        NodeEventsErrorsDBO dbo3 = create(3, "opKey",
                EventType.delete, "EventCategory2", "Path3", 0, 2, "defaultErrMsg");
        NodeEventsErrorsDBO dbo4 = create(4, "opKey2",
                EventType.props, "EventCategory3", "Path4", 0, 2, "defaultErrMsg");
        List<NodeEventsErrorsDBO> dbos = Lists.newArrayList(dbo1, dbo2, dbo3, dbo4);

        SqlParameterSource[] batch = SqlParameterSourceUtils.createBatch(dbos);
        simpleJdbcInsert.executeBatch(batch);

        dao.delete("EventCategory1", EventType.create, "Path1");
        dao.delete("EventCategory2", EventType.create, "Path2", "opKey");

        List<NodeEventsErrorsDBO> returnedDbos =
                jdbcTemplate.query("SELECT * FROM " + TABLE_NODE_EVENTS_ERRORS, getRowMapper());

        Assert.assertNotNull(returnedDbos);
        Assert.assertEquals(returnedDbos.size(), 2);

        returnedDbos.sort(Comparator.comparing(NodeEventsErrorsDBO::getErrorId));

        Assert.assertEquals(dbo3, returnedDbos.get(0));
        Assert.assertEquals(dbo4, returnedDbos.get(1));
    }

    @Test
    public void testDeleteByOperationKey() throws SQLException {
        NodeEventsErrorsDBO dbo1 = create(1, null, "EventCategory");
        NodeEventsErrorsDBO dbo2 = create(2, "opKey", "EventCategory");
        NodeEventsErrorsDBO dbo3 = create(3, "opKey", "EventCategory");
        NodeEventsErrorsDBO dbo4 = create(4, "opKey2", "EventCategory");
        List<NodeEventsErrorsDBO> dbos = Lists.newArrayList(dbo1, dbo2, dbo3, dbo4);

        SqlParameterSource[] batch = SqlParameterSourceUtils.createBatch(dbos);
        simpleJdbcInsert.executeBatch(batch);

        dao.delete("EventCategory", "opKey");

        List<NodeEventsErrorsDBO> returnedDbos =
                jdbcTemplate.query("SELECT * FROM " + TABLE_NODE_EVENTS_ERRORS, getRowMapper());

        Assert.assertNotNull(returnedDbos);
        Assert.assertEquals(2, returnedDbos.size());

        returnedDbos.sort(Comparator.comparing(NodeEventsErrorsDBO::getErrorId));

        Assert.assertEquals(dbo1, returnedDbos.get(0));
        Assert.assertEquals(dbo4, returnedDbos.get(1));
    }

    @Test
    public void testDeleteByOperationKeyAndEventType() throws SQLException {
        NodeEventsErrorsDBO dbo1 = create(1, null, EventType.update, "EventCategory");
        NodeEventsErrorsDBO dbo2 = create(2, "opKey", EventType.create, "EventCategory");
        NodeEventsErrorsDBO dbo3 = create(3, "opKey", EventType.create, "EventCategory");
        NodeEventsErrorsDBO dbo4 = create(4, "opKey2", EventType.delete, "EventCategory");
        List<NodeEventsErrorsDBO> dbos = Lists.newArrayList(dbo1, dbo2, dbo3, dbo4);

        SqlParameterSource[] batch = SqlParameterSourceUtils.createBatch(dbos);
        simpleJdbcInsert.executeBatch(batch);

        dao.delete("EventCategory", "opKey", EventType.create);

        List<NodeEventsErrorsDBO> returnedDbos =
                jdbcTemplate.query("SELECT * FROM " + TABLE_NODE_EVENTS_ERRORS, getRowMapper());

        Assert.assertNotNull(returnedDbos);
        Assert.assertEquals(2, returnedDbos.size());

        returnedDbos.sort(Comparator.comparing(NodeEventsErrorsDBO::getErrorId));

        Assert.assertEquals(dbo1, returnedDbos.get(0));
        Assert.assertEquals(dbo4, returnedDbos.get(1));
    }

    private NodeEventsErrorsDBO createUpdated(NodeEventsErrorsDBO dboToUpdate, int newErrorCount, int newLastErrorTime,
            String newErrMsg) {
        return NodeEventsErrorsDBO.builder()
                .errorCount(newErrorCount)
                .lastErrorTime(newLastErrorTime)
                .errorMessage(newErrMsg)

                .errorId(dboToUpdate.getErrorId())
                .eventCategory(dboToUpdate.getEventCategory())
                .operationKey(dboToUpdate.getOperationKey())
                .eventType(dboToUpdate.getEventType())
                .path(dboToUpdate.getPath())
                .firstErrorTime(dboToUpdate.getFirstErrorTime())
                .eventTime(dboToUpdate.getEventTime())
                .build();
    }

    private NodeEventsErrorsDBO create(int errorId, String operationKey, String eventCategory) {
        return create(errorId, operationKey, EventType.create, eventCategory);
    }

    private NodeEventsErrorsDBO create(int errorId, String operationKey, EventType eventType, String eventCategory) {
        return create(errorId, operationKey, eventType, eventCategory, "defaultPath");
    }

    private NodeEventsErrorsDBO create(int errorId, String operationKey, EventType eventType, String eventCategory,
            String path) {
        return create(errorId, operationKey, eventType, eventCategory, path, 0, 2, "defaultErrMsg");
    }

    private NodeEventsErrorsDBO create(int errorId, String operationKey, EventType eventType, String eventCategory,
            String path, int errorCount, int lastErrorTime, String errMsg) {
        return NodeEventsErrorsDBO.builder()
                .errorId(errorId)
                .errorCount(errorCount)
                .errorMessage(errMsg)
                .firstErrorTime(1)
                .lastErrorTime(lastErrorTime)
                .operationKey(operationKey)
                .eventCategory(eventCategory)
                .eventTime(0)
                .path(path)
                .eventType(eventType.code())
                .build();
    }

    private RowMapper<NodeEventsErrorsDBO> getRowMapper() {
        return (ResultSet rs, int i) -> NodeEventsErrorsDBO.builder()
                .errorId(rs.getLong("error_id"))
                .eventCategory(rs.getString("event_category"))
                .firstErrorTime(rs.getLong("first_error_time"))
                .lastErrorTime(rs.getLong("last_error_time"))
                .errorCount(rs.getInt("error_count"))
                .errorMessage(rs.getString("error_message"))
                .operationKey(rs.getString("operation_key"))
                .eventTime(rs.getLong("event_time"))
                .eventType(rs.getInt("event_type"))
                .path(rs.getString("path"))
                .build();
    }
}
