package org.artifactory.ui.rest.service.admin.security.signingkeys;

import org.artifactory.addon.AddonsManager;
import org.artifactory.addon.common.gpg.GpgKeyStore;
import org.artifactory.addon.ha.HaCommonAddon;
import org.artifactory.api.context.ArtifactoryContext;
import org.artifactory.api.context.ArtifactoryContextThreadBinder;
import org.artifactory.common.home.ArtifactoryHome;
import org.artifactory.rest.common.service.ArtifactoryRestResponse;
import org.artifactory.ui.rest.model.admin.security.signingkey.SignKey;
import org.jfrog.security.crypto.EncryptionWrapper;
import org.jfrog.security.crypto.PlainTextEncryptionWrapper;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.MockitoAnnotations;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import javax.ws.rs.core.Response;

import static org.mockito.ArgumentMatchers.eq;
import static org.mockito.Mockito.when;
import static org.testng.Assert.assertEquals;
import static org.testng.Assert.assertNull;

/**
 * @author Volodymyr Shulga
 */
public class VerifySigningKeyServiceTest {
    @Mock
    private AddonsManager addonsManager;
    @Mock
    private HaCommonAddon haCommonAddon;
    @Mock
    private ArtifactoryContext context;
    @Mock
    private ArtifactoryHome artifactoryHomeMock;
    @Mock
    private GpgKeyStore gpgKeyStore;

    private final EncryptionWrapper encryptionWrapper = new PlainTextEncryptionWrapper();
    private final String PASSPHRASE_NON_ENCRYPTED = "passphrase_non_encrypted";

    @BeforeMethod
    public void setup() throws Exception {
        MockitoAnnotations.initMocks(this);
        when(addonsManager.addonByType(HaCommonAddon.class)).thenReturn(haCommonAddon);
        when(haCommonAddon.getHostId()).thenReturn("4");
        when(context.beanForType(AddonsManager.class)).thenReturn(addonsManager);
        ArtifactoryContextThreadBinder.bind(context);
        ArtifactoryHome.bind(artifactoryHomeMock);
        when(artifactoryHomeMock.getArtifactoryEncryptionWrapper()).thenReturn(encryptionWrapper);

        Mockito.reset(gpgKeyStore);
    }

    @AfterMethod
    public void cleanup() {
        ArtifactoryHome.unbind();
        ArtifactoryContextThreadBinder.unbind();
    }

    @Test
    public void shouldSucceedForUnencryptedPassphrase() {
        when(gpgKeyStore.hasPublicKey()).thenReturn(true);
        when(gpgKeyStore.hasPrivateKey()).thenReturn(true);
        when(gpgKeyStore.verify(eq(PASSPHRASE_NON_ENCRYPTED))).thenReturn(true);

        final SignKey signKey = new SignKey();
        signKey.setPublicKeyInstalled(true);
        signKey.setPrivateKeyInstalled(true);

        signKey.setPassPhrase(PASSPHRASE_NON_ENCRYPTED);

        final ArtifactoryRestResponse<String> response = new ArtifactoryRestResponse<>();
        VerifySigningKeyService signingKeyService = new VerifySigningKeyService(gpgKeyStore);
        signingKeyService.execute(signKey, response);
        assertEquals(response.getResponseCode(), Response.Status.OK.getStatusCode());
        assertNull(response.getFeedbackMsg().getError());
        assertEquals(response.getFeedbackMsg().getInfo(), "Successfully verified keys and passphrase");
    }

    @Test
    public void shouldFailForNotSetPublicKey() {
        when(gpgKeyStore.hasPublicKey()).thenReturn(false);

        final SignKey signKey = new SignKey();
        signKey.setPassPhrase(PASSPHRASE_NON_ENCRYPTED);

        final ArtifactoryRestResponse<String> response = new ArtifactoryRestResponse<>();
        VerifySigningKeyService signingKeyService = new VerifySigningKeyService(gpgKeyStore);
        signingKeyService.execute(signKey, response);
        assertEquals(response.getResponseCode(), Response.Status.OK.getStatusCode());
        assertNull(response.getFeedbackMsg().getInfo());
        assertEquals(response.getFeedbackMsg().getError(), "No public key installed");
    }

    @Test
    public void shouldFailForNotSetPrivateKey() {
        when(gpgKeyStore.hasPublicKey()).thenReturn(true);
        when(gpgKeyStore.hasPrivateKey()).thenReturn(false);

        final SignKey signKey = new SignKey();
        signKey.setPassPhrase(PASSPHRASE_NON_ENCRYPTED);

        final ArtifactoryRestResponse<String> response = new ArtifactoryRestResponse<>();
        VerifySigningKeyService signingKeyService = new VerifySigningKeyService(gpgKeyStore);
        signingKeyService.execute(signKey, response);
        assertEquals(response.getResponseCode(), Response.Status.OK.getStatusCode());
        assertNull(response.getFeedbackMsg().getInfo());
        assertEquals(response.getFeedbackMsg().getError(), "No private key installed");
    }
}